/*
 * $Id: fcp.c,v 1.34 2003/03/01 22:38 jaime Exp $
 *
 * FCP module
 *
 *
 * Copyright (C) 2001-2003 Fhg Fokus
 *
 * This file is part of ser, a free SIP server.
 *
 * ser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * ser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <stdio.h>
#include <time.h>

#include "../../dprint.h"
#include "../../parser/msg_parser.h"
#include "../tm/t_funcs.h"
#include "../../error.h"
#include "../../ut.h"      /* q_memchr */

#include "fcp_mod.h"
#include "fcp.h"
#include "text_ops.h"

/* 
   Replaces Header in sip_msg. It will replace the header pointed
   by <key> with the header pointed by <str>. Both are NULL
   terminated strings
*/

int replace_header(struct sip_msg* msg, char* key, char* str, char *where)
{
  struct lump* l;
  regmatch_t pmatch;
  char* s;
  int len;
 
  int    status=-1;

  if(!where) return -1;
 
  /* Searches for <key> from the beginning of msg */

  DBG("Replace_header: searching for:%s\nIts length: %d\n",
	 key, strlen(key));

  status = match(where, key, &pmatch);
  if(status!=0) {
    DBG("Replace_header. Regexec returned %d\n", status);
    return -1;
  }

  DBG("replace_header: Offset start= %d, offset end= %d\n", pmatch.rm_so, pmatch.rm_eo);

  if (pmatch.rm_so!=-1){
      
    if ((l=del_lump(&msg->add_rm, pmatch.rm_so,
		    pmatch.rm_eo-pmatch.rm_so, 0))==0) {
      LOG(L_ERR, "Error deleting lump\n");
      return -1;
    }
    len=strlen(str);
    s=pkg_malloc(len);
    if (s==0){
      LOG(L_ERR, "ERROR: replace_header: mem. allocation failure\n");
      return -1;
    }
    memcpy(s, str, len); 
    return insert_new_lump_after(l, s, strlen(str), 0)?1:-1;
  }

  return 1;

}

char *get_header(struct sip_msg *msg)
{
#ifdef SCRATCH
	return msg->orig+msg->first_line.len;
#else
	return msg->buf+msg->first_line.len;
#endif
}


int replace_all(struct sip_msg* msg, char* key, char* str)
{


	struct lump* l;
	regmatch_t pmatch;
	char* s;
	int len;
	char* begin;
	int off;
	int ret;

	begin=get_header(msg); /* msg->orig previously .. uri problems */
	ret=-1; /* pessimist: we will not find any */
	len=strlen(str);

	while (begin<msg->buf+msg->len 
				&& match(begin, key, &pmatch)==0) {
		off=begin-msg->buf;
		if (pmatch.rm_so==-1){
			LOG(L_ERR, "ERROR: replace_all: offset unknown\n");
			return -1;
		}
		if ((l=del_lump(&msg->add_rm, pmatch.rm_so+off,
						pmatch.rm_eo-pmatch.rm_so, 0))==0) {
			LOG(L_ERR, "ERROR: replace_all: del_lump failed\n");
			return -1;
		}
		s=pkg_malloc(len);
		if (s==0){
			LOG(L_ERR, "ERROR: replace_all: mem. allocation failure\n");
			return -1;
		}
		memcpy(s, str, len); 
		if (insert_new_lump_after(l, s, len, 0)==0){
			LOG(L_ERR, "ERROR: replace_all: could not insert new lump\n");
			pkg_free(s);
			return -1;
		}
		/* new cycle */
		begin=begin+pmatch.rm_eo;
		ret++;
	} /* while found ... */
	if (ret==-1) return ret;
	else return ret+1;
}


/* Replaces Contact header of SIP message */
 
int replace_contact(struct sip_msg *msg, struct session *s) {
 
  str old;
  char *buf, *p, *buf2;
  int len;

  if(!msg->contact) return -1;
  if(!s) return -1;

  if (parse_headers(msg, HDR_CONTACT, 0)==-1){
    LOG(L_ERR, "replace_contact: No Contact field present!\n");
    return -1;
  }

  old = msg->contact->body;

  len = old.len;

  DBG("replace_contact: Allocating memory for %d bytes\n", len);  

  buf = pkg_malloc( len+1 );
  if(!buf) {
    LOG(L_ERR,"ERROR:replace_contact: out of memory! \n");
    return -1;
  }
  
  p = buf;
  append_str( p, old.s , old.len);
  *(p++) = '\0';

  printf("replace_contact(): Old contact=%s\n", buf);


  lock_get(&sl->lock);

  /* Build the new Contact field */   

  len = /* 9, "Contact: " + */ 4 /* "sip:" */ + \
    s->fw_ip1.len + \
    1 /*":"*/ + s->fw_port1.len /* port length */ + 2/*"<>"*/ \
    /* + CRLF_LEN */ + 1; /* for "\0"? */

  buf2 = pkg_malloc( len + 1 );
  if(!buf2) {
    LOG(L_ERR,"ERROR:replace_contact: out of memory! \n");
    goto error;
  }
  
  p = buf2;
  //  append_str( p, "Contact: " , 9);
  *(p++) = '<';
  append_str( p, "sip:", 4);
  append_str( p, s->fw_ip1.s, s->fw_ip1.len);
  *(p++) = ':';
  append_str( p, s->fw_port1.s, s->fw_port1.len);
  *(p++) = '>';
  //  append_str( p, CRLF, CRLF_LEN);
  *(p++) = '\0';  

  lock_release(&sl->lock);

  printf("replace_contact() New header: %s\nLength=%d\n", buf2, len);

  if(replace_header(msg, buf, buf2, msg->buf)==-1) {
    LOG(L_ERR, "Could not replace header\n");
    goto error;
  }

  return 1;

 error:
  lock_release(&sl->lock);
  if(buf) pkg_free(buf);
  if(buf2) pkg_free(buf2);
  return -1;
}

/**************************************** 
   Replaces IP addresses and ports in SDP
   message with the info provided by an
   FCP server. Returns number of occurences.

*****************************************/

int replace_sdp(struct sip_msg *msg, struct session *ses) {

  int len, count, count2, old_content_length, diff;
  char *ptr, *buf, *buf2;
  char tmp[6];

  char *string_end;

  diff = 0;  


  old_content_length = string_to_int(msg->content_length->body.s);

  DBG("replace_sdp: Content length=%d\n", old_content_length);
  if (old_content_length == 0) {
    LOG(L_ERR, "Message Body not found!!\n");
    return -1;
  }


  /* 
     Search for "IP4 <host>" and "audio <port>" strings 
     within the sip message. Replace their values accordingly
  */

  len = 7/* "IN IP4 " */ + ses->local_sdp.ip.len;
  buf = pkg_malloc( len+1 ); /* extra character for \0? */
  if(!buf) {
    LOG(L_ERR,"ERROR:extcmd_add_contact: out of memory! \n");
    return -1;
  }
  
  ptr = buf;
  append_str( ptr, "IN IP4 " , 7);
  append_str( ptr, ses->local_sdp.ip.s, ses->local_sdp.ip.len);
  *(ptr++) = '\0';


  len = 7 /* "IN IP4 " */ + ses->fw_ip1.len;
  buf2 = pkg_malloc( len+1 ); /* extra character for \0? */
  if(!buf2) {
    LOG(L_ERR,"ERROR:replace_sdp: out of memory! \n");
    return -1;
  }
  
  ptr = buf2;
  append_str( ptr, "IN IP4 " , 7);
  append_str( ptr, ses->fw_ip1.s, ses->fw_ip1.len);
  *(ptr++) = '\0';


  string_end = get_header(msg);

  DBG("Replace_SDP: Replacing %s with %s\n", buf, buf2);

  count = 0;

  if((count=replace_all(msg, buf, buf2))==-1) {
    LOG(L_ERR, "Replace_SDP. Could not replace IP\n");
    goto cont;
  }

  diff = count * (strlen (buf) - strlen (buf2));


 cont:

  len = 6 /* "audio " */ + ses->local_sdp.port.len;
  buf = pkg_malloc( len+1 ); /* extra character for \0? */
  if(!buf) {
    LOG(L_ERR,"ERROR:replace_sdp: out of memory! \n");
    return -1;
  }
  
  ptr = buf;
  append_str( ptr, "audio " , 6);
  append_str( ptr, ses->local_sdp.port.s, ses->local_sdp.port.len);
  *(ptr) = '\0';


  len = 6 /* "audio " */ + ses->fw_port1.len;
  buf2 = pkg_malloc( len+1 ); /* extra character for \0? */
  if(!buf2) {
    LOG(L_ERR,"ERROR:replace_sdp: out of memory! \n");
    return -1;
  }
  
  ptr = buf2;
  append_str( ptr, "audio " , 6);
  append_str( ptr, ses->fw_port1.s, ses->fw_port1.len);
  *(ptr) = '\0';


  DBG("Replace_SDP: Replacing %s with %s\n", buf, buf2);

  count2 = 0;

  if((count2+=replace_all(msg, buf, buf2))==-1) {
    LOG(L_ERR, "Replace_SDP. Could not replace media port\n");
    goto error;
  }

  diff += count2 * (strlen (buf) - strlen (buf2));

  /**************************
    Replace Content-Length 
  ***************************/

  /* Get old content length */

  len = msg->content_length->name.len+2 /* Content-Length: */ + msg->content_length->body.len;

  buf = pkg_malloc( len+1 ); /* extra character for \0? */
  if(!buf) {
    LOG(L_ERR,"ERROR:replace_sdp: out of memory! \n");
    return -1;
  }
  
  //  old_content_length = string_to_int(msg->content_length->body.s);

  ptr=buf;
  append_str(ptr, msg->content_length->name.s, msg->content_length->name.len);
  append_str(ptr, ": ", 2);
  append_str(ptr, msg->content_length->body.s, msg->content_length->body.len);
  *(ptr++)='\0';

  DBG("replace_sdp: Previous Content-Length: %s\n", buf);

  /* Get new content length */

  old_content_length = old_content_length + diff;

  int_to_char(tmp, old_content_length, sizeof(int));

  DBG("New Content-Length: %s\n", tmp);

  len = msg->content_length->name.len+2 /*: */ + 5;

  buf2 = pkg_malloc( len+1 ); /* extra character for \0? */
  if(!buf2) {
    LOG(L_ERR,"ERROR:replace_sdp: out of memory! \n");
    return -1;
  }

  ptr = buf2;
  append_str(ptr, msg->content_length->name.s, msg->content_length->name.len);
  append_str(ptr, ": ", 2);
  append_str(ptr, tmp, strlen(tmp));
  *(ptr++) = '\0';
  msg->content_length->name.len = strlen(tmp);

  DBG("Replace_SDP: Replacing %s with %s\n", buf, buf2);

  if(replace_all(msg, buf, buf2)==-1) {
    LOG(L_ERR, "Replace_SDP. Could not replace Content Length\n");
    goto error;
  }

 error:
    if(buf) pkg_free(buf);
    if(buf2) pkg_free(buf2);

    return count; 

}

/***************************************************

   Adds "rport" to the via header. That field will
be completed by the next proxy with the port the
request was sent from. Need to do the same for
"received"?

Proxy1:

Via: SIP/2.0/UDP 10.1.1.1:4540;received;rport

Proxy2:

Via: SIP/2.0/UDP 10.1.1.1:4540;received=68.44.20.1;rport=9988


****************************************************/

int replace_via(struct sip_msg *msg, struct session *s) {

  char *ptr;
  int len;

  if(msg->via1) {

    DBG("replace_sdp: Via present\n");

    /* Build the new Via using "received" and "rport" values */   


    len = /* 1 ";"*/ msg->add_to_branch_len + 6 /* ";rport"*/;
  
    /*
    buf = pkg_malloc( len + 1);
    if(!buf) {
      LOG(L_ERR,"ERROR:extcmd_add_contact: out of memory! \n");
      return -1;
    }
    */  

    ptr = msg->add_to_branch_s + msg->add_to_branch_len;
    append_str(ptr, ";rport", 6);
    *(ptr)='\0';

    msg->add_to_branch_len=ptr-msg->add_to_branch_s;

    /*
    p = buf;
    append_str( p, via_param1 , via_param1_len);
    append_str( p, ";" , 1);
    append_str( p, "received=" , 9);
    append_str( p, s->fw_ip1.s, s->fw_ip1.len);
    *(p++) = ';';
    append_str( p, "rport=" , 6);
    append_str( p, s->fw_port1.s, s->fw_port1.len);
    */
    //    *(p++) = ';';


    //    *(p++) = '\0';  

    DBG("New header: %s\n", msg->add_to_branch_s);

    //    if(buf) {
    //      pkg_free(buf);
    //    }

    /*
    if(replace_header(msg, params1, buf, msg->buf)==-1) {
      LOG(L_ERR, "Could not insert "received" param\n");
      return -1;
    }
    msg->via1->params.len=strlen(buf);
    */

   }

    /*
  if(msg->via2) {
    host2= msg->via2->host.s;
    port2= msg->via2->port_str.s;
    params2= msg->via2->params.s;
    hdr2= msg->via2->hdr.s;

    DBG("Replace Via: Host2 = %s, Port2 = %s,Params2 = %s, HDR = %s\n",
	   host2, port2, params2, hdr2);

  }

    */

  return 1;
}

/* Not working properly */

int add_contact(struct sip_msg* msg , str* to_uri)
{
  struct lump_rpl *lump;
  char *buf, *p;
  int len;
  
  len = 9 /*"Contact: "*/ + to_uri->len + 2/*"<>"*/ + CRLF_LEN;
  
  buf = pkg_malloc( len );
  if(!buf) {
    LOG(L_ERR,"ERROR:extcmd_add_contact: out of memory! \n");
    return -1;
  }
  
  p = buf;
  append_str( p, "Contact: " , 9);
  *(p++) = '<';
  append_str( p, to_uri->s, to_uri->len);
  *(p++) = '>';
  append_str( p, CRLF, CRLF_LEN);
  
  lump = build_lump_rpl( buf , len );
  if(!lump) {
    LOG(L_ERR,"ERROR:extcmd_add_contact: unable to build lump_rpl! \n");
    pkg_free( buf );
    return -1;
  }
  add_lump_rpl( msg , lump );
  
  pkg_free(buf);
  return 1;
}
