/*
 * $Id: parser.c,v 1.5 2003/ 12:23:53 jaime Rel $
 *
 * Copyright (C) 2001-2003 Fhg Fokus
 *
 * This file is part of ser, a free SIP server.
 *
 * ser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * ser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "../../str.h"
#include "../../parser/msg_parser.h"

#include "parser.h"
#include "text_ops.h"



/* 
   Gets the media info from the sdp message from the
   request and the response and puts them in
   a struct that will be used to query the firewall
*/

int parse_media_info(struct sip_msg *request, struct sip_msg *reply, struct session *session) {

  str sdp;


  if(reply == NULL) {
    printf("No reply present. Trying to parse request then...\n");
    goto request_only;
  }



  /* get the SDP part of the response */
  
  sdp.s = get_body( reply );
  if (sdp.s==0) {
    LOG(L_ERR,"ERROR:parse_media_info cannot extract sdp from response!\n");
    goto error;
  }
  sdp.len = (int)reply->content_length->parsed;

  /*
    Get the media information from the response.
    Starting with the SRC's IP address...
  */

  lock_get(&sl->lock);

  session->remote_sdp.ip.s = get_char_value(sdp.s, "IP4");

  if(session->remote_sdp.ip.s == NULL) {
    LOG(L_ERR, "parse_media_info: Error getting remote IP from SDP\n");
    goto error;
  }

  session->remote_sdp.ip.len = strlen(session->remote_sdp.ip.s);

  printf("parse_media_info: Remote HOST = %s. Len = %d\n", session->remote_sdp.ip.s, session->remote_sdp.ip.len);

  /* Get the SRC's port */

  session->remote_sdp.port.s = get_char_value(sdp.s, "audio");

  if(session->remote_sdp.port.s == NULL) {
    LOG(L_ERR, "parse_media_info: Error getting remote Port from SDP\n");
    goto error;
  }

  session->remote_sdp.port.len = strlen(session->remote_sdp.port.s);

  printf("parse_media_info: Remote PORT = %s.Len = %d\n", session->remote_sdp.port.s, session->remote_sdp.port.len);
  




  /* Now get the SDP part of the original request */
  
 request_only:

  if(request == NULL) {
    printf("Request message not present. No SDP to parse...\n");
    goto error;
  }
  
  sdp.s = get_body( request );
  if (sdp.s==0) {
    LOG(L_ERR,"ERROR:parse_media_info cannot extract sdp from msg!\n");
    goto error;
  }
  sdp.len = (int)request->content_length->parsed;

  /*
    Get the media information from the request.
    Starting with the DST's IP address...
  */


  session->local_sdp.ip.s = get_char_value(sdp.s, "IP4");

  if(session->local_sdp.ip.s == NULL) {
    LOG(L_ERR, "parse_media_info: Error getting local IP from SDP\n");
    goto error;
  }

  session->local_sdp.ip.len = strlen(session->local_sdp.ip.s);

  printf("parse_media_info: Local HOST = %s. Len = %d\n", session->local_sdp.ip.s, session->local_sdp.ip.len);

  /* Get the DST's port */

  session->local_sdp.port.s = get_char_value(sdp.s, "audio");

  if(session->local_sdp.port.s == NULL) {
    LOG(L_ERR, "parse_media_info: Error getting local Port from SDP\n");
    goto error;
  }

  session->local_sdp.port.len = strlen(session->local_sdp.port.s);


  printf("parse_media_info: Local PORT = %s.Len = %d\n", session->local_sdp.port.s, session->local_sdp.port.len);


  lock_release(&sl->lock);

  return 1; /* Successful */


 error:
  lock_release(&sl->lock);
  if(session->remote_sdp.ip.s!=NULL) shm_free(session->remote_sdp.ip.s);
  if(session->remote_sdp.port.s!=NULL) shm_free(session->remote_sdp.port.s);
  if(session->local_sdp.ip.s!=NULL) shm_free(session->local_sdp.ip.s);
  if(session->local_sdp.port.s!=NULL) shm_free(session->local_sdp.port.s);
  return -1;
}



/*
  Get the nat response and process it. Put useful
  info into session struct, mainly fw_ip1, fw_port1,
  seq
*/

int parse_fcp_info(char *s, struct session *session) {

  char *string_start, *buf;
  char *string_end, *ptr;
  int end, len;


  /* IP information from FCP server */

  end = get_offset_end(s, "IP");
  string_start = skip_delims(s + end);
  string_end = skip_chars(string_start);
	
  len = string_end - string_start;
  
  buf = shm_malloc( len+1 ); /* extra character for \0? */
  if(!buf) {
    LOG(L_ERR,"ERROR:parse_fcp_info: out of memory! \n");
    goto error;
  }

  printf("parse_fcp_info()");
  get_lock2(&sl->lock);

  session->fw_ip1.s = buf;
  memcpy( session->fw_ip1.s, string_start , len);
  *(session->fw_ip1.s + len) = '\0';
  session->fw_ip1.len=len;
  DBG("parse_fcp_info: FCP Rply Host = %s. Length: %d\n", session->fw_ip1.s, len);

  /* Port information */

  end = get_offset_end(s, "PORT");
  string_start = skip_delims(s + end);
  string_end = skip_chars(string_start);
	
  len = string_end - string_start;
  
  buf = shm_malloc( len+1 ); /* extra character for \0? */
  if(!buf) {
    LOG(L_ERR,"ERROR:parse_fcp_info: out of memory! \n");
    goto error;
  }
 
  session->fw_port1.s = buf;
  memcpy( session->fw_port1.s, string_start , len);
  *(session->fw_port1.s + len) = '\0';
  session->fw_port1.len = len;
  DBG("parse_fcp_info: FCP Rply port = %s. Length: %d\n", session->fw_port1.s, len);



  /* Seq information */

  end = get_offset_end(s, "SEQ");
  string_start = skip_delims(s + end);
  string_end = skip_chars(string_start);
	
  len = string_end - string_start;
  
  buf = shm_malloc( len+1 ); /* extra character for \0? */
  if(!buf) {
    LOG(L_ERR,"ERROR:parse_fcp_info: out of memory! \n");
    goto error;
  }
 
  ptr = buf;
  memcpy( ptr, string_start , len);
  *(ptr + len) = '\0';
  session->seq = string_to_int(buf);

  if(buf)
    shm_free(buf);

  DBG("parse_fcp_info: SEQ = %d. Length: %d\n", session->seq, len);

  release_lock2(&sl->lock);

  return 1; /* Successful */

 error:
  release_lock2(&sl->lock);
  if(session->fw_ip1.s!=NULL) shm_free(session->fw_ip1.s);
  if(session->fw_port1.s!=NULL) shm_free(session->fw_port1.s);
  return -1;
}

/* 
   Checks whether <message> is of type <method>.
   <method> is a null terminated string.
   Returns 1 if <message> is of type <method>,
   0 if false. If request=1, looks for <method>.
   If request=0 looks for SIP/2.0 <method>.
*/

int is_method(char *message, char *method, int request) {

  int len, offset;
  char *string_start;

  len = strlen(method);

  DBG("is_method: Checking if is method %s\n", method);

  if(!request) {
    offset = get_offset_end(message, "SIP/2.0");
    string_start = skip_delims(message + offset);
  }

  /* Else is a response */

  else string_start = message;

  return (strncmp(string_start, method, (size_t)len)==0);

}

/* Parse call id into session struct */

int parse_callid (struct sip_msg *msg, struct session *s) {

  str header;
  char *buf = NULL;

  /* Starting from the local Contact */

  if(!msg->callid) {
    printf("parse_callid: No Callid Present in message!\n");
    goto error;
  }
 
  if(s == NULL) {
    printf("parse_callid: No session parsed!\n");
    goto error;    
  }

  header = msg->callid->body;
  
  buf = shm_malloc(header.len+1);

  if(!buf) {
    LOG(L_ERR,"ERROR:parse_callid: out of memory! \n");
    goto error;
  }


  memcpy(buf, header.s, header.len);
  buf[header.len]='\0';
  printf("parse_callid: Callid: %s\n", buf);

  get_lock2(&sl->lock);

  s->callid.s = buf;
  s->callid.len = strlen(s->callid.s);

  printf("parse_callid() callid=%s. Len=%d\n", s->callid.s, s->callid.len);

  release_lock2(&sl->lock);
  return 1;

 error:
  release_lock2(&sl->lock);
  if(buf) shm_free(buf);
  return -1;

}


/* Parse the info contained in the Contact if exists
   or, in its default, in the last Via
*/

int parse_outbound_info (struct sip_msg* msg, struct session *s) {

  str header;
  char *buf, *host, *port;

  printf("parse_outbound_info:\n\n%s\n", msg->buf);

  buf = NULL;

  if(!msg->contact) {
    printf("parse_outbound_info: No Contact Header Present!\n");
    goto get_via;
  }
 
  header = msg->contact->body;
  
  buf = shm_malloc(header.len+1);

  if(buf == NULL) {
    LOG(L_ERR,"ERROR:parse_outbound_info: out of memory! \n");
    goto error;
  }

  memcpy(buf, header.s, header.len);
  buf[header.len]='\0';
  printf("parse_outbound_info: Contact: %s\n", buf);

  s->local_contact.ip.s = get_char_value(buf, "sip");
  s->local_contact.ip.len = strlen(s->local_contact.ip.s);

  s->local_contact.port.s = get_char_value(buf+s->local_contact.ip.len, ":");
  s->local_contact.port.len = strlen(s->local_contact.port.s);

  printf("parse_outbound_info: IP: %s\n", s->local_contact.ip.s);
  printf("parse_outbound_info: Port: %s\n", s->local_contact.port.s);

  if(buf) shm_free(buf);

  return 1;

 get_via:
  /* Segmentation fault problem */
  /*
  buf = get_char_value(msg->buf, "SIP/2.0/UDP");
  if(buf == NULL) {
    printf("No Via Present!\n");
    return -1;
  }
  */

  /*
  buf = msg->buf;

  buf = get_char_value(buf, "Via");
  printf("After Via: \n", buf);
  s->local_contact.ip.s = get_char_value(msg->buf, buf);
  s->local_contact.ip.len = strlen(s->local_contact.ip.s);

  s->local_contact.port.s = get_char_value(buf, s->local_contact.ip.s);
  s->local_contact.port.len = strlen(s->local_contact.port.s);

  printf("parse_outbound_info: IP: %s\n", s->local_contact.ip.s);
  printf("parse_outbound_info: Port: %s\n", s->local_contact.port.s);

  if(buf) shm_free(buf);
  */

  if(!msg->via1) {
    printf("parse_outbound_info: No Via present either. What can I do with this packet?\n");
    return -1;
  }

  host = shm_malloc(msg->via1->host.len+1);

  if(!host) {
    LOG(L_ERR,"ERROR:parse_outbound_info: out of memory! \n");
    goto error;
  }

  memcpy(host, msg->via1->host.s, msg->via1->host.len);
  host[msg->via1->host.len] = '\0';
  s->local_contact.ip.s = host;
  s->local_contact.ip.len = msg->via1->host.len;

  port = shm_malloc(msg->via1->port_str.len+1);

  if(!port) {
    LOG(L_ERR,"ERROR:parse_outbound_info: out of memory! \n");
    goto error;
  }

  memcpy(port, msg->via1->port_str.s, msg->via1->port_str.len);
  port[msg->via1->port_str.len] = '\0';
  s->local_contact.port.s = port;
  s->local_contact.port.len = strlen(s->local_contact.port.s);
  
  printf("parse_outbound_info: IP: %s\n", s->local_contact.ip.s);
  printf("parse_outbound_info: Port: %s\n", s->local_contact.port.s);

  return 1;
 
 error:
  if (buf) shm_free(buf);
  return -1;
}

/****************************************

 Parse the contact information from the request and the response.
Namely, local ip, local port, remote ip, remote port.

******************************************/
int parse_contact_info (struct sip_msg *rq, struct sip_msg *rpl, struct session *s) {

  str header;
  char *buf;

  buf = NULL;

  if(rq == NULL) {
    printf("No request present. Trying to parse reply then...\n");
    goto reply_only;
  }


  /* Starting from the local Contact */

  if(!rq->contact) {
    printf("parse_contact_info: No Contact Header Present in request!\n");
    goto error;
  }
 
  header = rq->contact->body;
  

  buf = shm_malloc(header.len+1);

  if(!buf) {
    LOG(L_ERR,"ERROR:parse_contact_info: out of memory! \n");
    goto error;
  }

  memcpy(buf, header.s, header.len);
  buf[header.len]='\0';
  printf("parse_contact_info: Contact: %s\n", buf);

  get_lock2(&sl->lock);

  s->local_contact.ip.s = get_char_value(buf, "sip");
  s->local_contact.ip.len = strlen(s->local_contact.ip.s);

  s->local_contact.port.s = get_char_value(buf+s->local_contact.ip.len, ":");
  s->local_contact.port.len = strlen(s->local_contact.port.s);

  printf("parse_contact_info: local IP: %s\n", s->local_contact.ip.s);
  printf("parse_contact_info: local Port: %s\n", s->local_contact.port.s);


  /* Now the remote Contact */

 reply_only:

  if(rpl == NULL) {
    printf("No contact in reply present...No contact to parse\n");
    goto error;
  }

  if(!rpl->contact) {
    printf("parse_contact_info: No Contact Header Present in reply!\n");
    goto error;
  }
 
  header = rpl->contact->body;

  if(buf) shm_free(buf);

  buf = shm_malloc(header.len+1);

  if(!buf) {
    LOG(L_ERR,"ERROR:parse_contact_info: out of memory! \n");
    goto error;
  }

  memcpy(buf, header.s, header.len);
  buf[header.len]='\0';
  printf("parse_contact_info: Contact: %s\n", buf);

  s->remote_contact.ip.s = get_char_value(buf, "sip");
  s->remote_contact.ip.len = strlen(s->local_contact.ip.s);

  s->remote_contact.port.s = get_char_value(buf+s->remote_contact.ip.len, ":");
  s->remote_contact.port.len = strlen(s->remote_contact.port.s);

  printf("parse_contact_info: remote IP: %s\n", s->remote_contact.ip.s);
  printf("parse_contact_info: remote Port: %s\n", s->remote_contact.port.s);

  if(buf) shm_free(buf);

  release_lock2(&sl->lock);

  return 1;

 error:
  release_lock2(&sl->lock);
  if(buf) shm_free(buf);
  return -1;
}
