/*
 * $Id: session.h,v 1.5 2003/ 12:23:53 jaime Rel $
 *
 * Copyright (C) 2001-2003 Fhg Fokus
 *
 * This file is part of ser, a free SIP server.
 *
 * ser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * ser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef session_h
#define session_h

#include <stdio.h>
#include <pthread.h>
#include "../../str.h"
#include "../../mem/mem.h"
#include "../../locking.h"

#define SESSION_INACTIVE 0
#define SESSION_ACTIVE 1
#define SESSION_IN_PROGRESS 2
#define SESSION_ERROR 3

#define PROTO_UDP 1
#define PROTO_TCP 2
#define PROTO_TLS 3

#define MAX_SESSIONS 10

int pdes[2];

struct session_list {
  struct session *last_session;
  int num_sessions;
  gen_lock_t lock;
} *sl;

//pthread_mutex_t mutx = PTHREAD_MUTEX_INITIALIZER;
pthread_mutex_t mutx;

struct m_contact {
  str body;
  str ip;
  str port;
};

struct m_sdp {
  str ip;
  str port;
  str media_type;
};

struct session {

  str callid; /* To match sessions */
  str gw_callid; /* Replaced callid */
  int proto; /* PROTO_UDP, PROTO_TCP, PROTO_TLS */

  struct m_contact local_contact; /* Information
				     contained in
				     contact field
				     of local machine
				  */
  struct m_sdp local_sdp; /* Information contained
			     in SDP of local machine
			  */

  int state;  /* State of the connection: INACTIVE
		 PROGRESS, ACTIVE, ERROR
	      */

  struct m_contact remote_contact; /* Information
				     contained in
				     contact field
				     of remote machine
				  */
  struct m_sdp remote_sdp; /* Information contained
			     in SDP of remote machine
			  */

  str fw_ip1; /* Reserved IP in external interface of FW/NAT */
  str fw_ip2; /* Another for sessions with Contact!=SDP */
  str fw_port1; /* Reserved PORT in external interface of FW/NAT */
  str fw_port2; /* Another for sessions with Contact!=SDP */

  int seq; /* FCP request identifier */
  int fcp_timer; /* How long rules last for in FW/NAT */

  str tag; /* Tag in To to identify session */

  struct session *next; /* Next in the chain */
  struct session *previous; /*  Previous in the chain */
};

static inline void get_lock2(gen_lock_t* lock) {
  get_lock(lock);
  printf("GOT lock...\n");
}


static inline void release_lock2(gen_lock_t* lock) {
  release_lock(lock);
  printf("RELEASED lock...\n");
}


/* Create a new Session struct initialised to 0 */
struct session* create_session();
int destroy_session(struct session *deleted);
struct session* append_session (struct session *s, struct session *last_s);
struct session* delete_session(struct session *s);
struct session* find_session(struct session *s, str *callid);
struct session* find_session_from_remote (struct session *last, str *ip, str *port);
struct session* find_session_from_local (struct session *last, str *ip, str *port);
struct session* find_session_from_fw (struct session *last, str *ip, str *port);struct session* replace_session (struct session *old, struct session *new);

void send_last_session(struct session *last);

#endif
