/*
 * $Id: tcpClient.c,v 1.5 2003/ 12:23:53 jaime Rel $
 *
 * Copyright (C) 2001-2003 Fhg Fokus
 *
 * This file is part of ser, a free SIP server.
 *
 * ser is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * ser is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <stdio.h>
#include <unistd.h> /* close */
#include <string.h>
#include "fcp_pa.h"
#include "tcpClient.h"

int openConnection(char *hostip, u_int port) {

  char *func="openConnection";
  int sd, rc;
  struct sockaddr_in localAddr, servAddr;
  struct hostent *h;
  
  h = gethostbyname(hostip);
  if(h==NULL) {
    LOG(L_ERR, "%s: unknown host '%s'\n",func, hostip);
    return -1;
  }

  servAddr.sin_family = h->h_addrtype;
  memcpy((char *) &servAddr.sin_addr.s_addr, h->h_addr_list[0], h->h_length);
  servAddr.sin_port = htons(port);

  /* create socket */
  sd = socket(AF_INET, SOCK_STREAM, 0);
  if(sd<0) {
    perror("cannot open socket ");
    return -1;
  }

  /* bind any port number */
  localAddr.sin_family = AF_INET;
  localAddr.sin_addr.s_addr = htonl(INADDR_ANY);
  localAddr.sin_port = htons(0);
  
  rc = bind(sd, (struct sockaddr *) &localAddr, sizeof(localAddr));
  if(rc<0) {
    LOG(L_ERR, "%s: cannot bind port TCP %u\n",func ,port);
    perror("error ");
    return -1;
  }

 				
  /* connect to server */
  rc = connect(sd, (struct sockaddr *) &servAddr, sizeof(servAddr));
  if(rc<0) {
    LOG(L_ERR, "openConnection: Error connecting to socket\n");
    perror("cannot connect ");
    return -1;
  }

return sd;
  
}

int closeConnection(int sd) {

  char *func="closeConnection";

  /* In the future, we may want to check a database for open connections */

  if(sd<0) {
    LOG(L_ERR, "%s: Error closing socket\n", func);
    perror("closeConnection: Cannot close");
    return(-1);
  } 

  close(sd);

  return 0;

}
